import 'package:adeptarch_base/src/filters/filter.dart';
import 'package:adeptutils/adeptutils.dart';

export 'package:adeptarch_base/src/filters/text_filter.dart';
export 'package:adeptarch_base/src/filters/filter_controller.dart';
export 'package:adeptarch_base/src/filters/date_filter.dart';
export 'package:adeptarch_base/src/filters/enum_filter.dart';
export 'package:adeptarch_base/src/filters/number_filter.dart';
export 'package:adeptarch_base/src/filters/bool_filter.dart';
export 'package:adeptarch_base/src/filters/filter.dart';

part 'filters.g.dart';

///Run flutter packages pub run build_runner build in order to changes take effect
@JsonSerializable()
class Filters extends Jsonable<Filters> {
  String? key;
  late List<TextFilter> textFilters;
  late List<NumberFilter> numberFilters;
  late List<BoolFilter> boolFilters;
  late List<DateFilter> dateFilters;
  late List<EnumFilter> enumFilters;

  Filters({this.key}) {
    clear();
  }

  Filters clear() {
    textFilters = [];
    numberFilters = [];
    boolFilters = [];
    dateFilters = [];
    enumFilters = [];
    return this;
  }

  bool get isEmpty =>
      textFilters.isEmpty &&
      numberFilters.isEmpty &&
      boolFilters.isEmpty &&
      dateFilters.isEmpty &&
      enumFilters.isEmpty;

  Filters addFilter(Filter filter) {
    if (filter is TextFilter) {
      textFilters.add(filter);
    }
    if (filter is NumberFilter) {
      numberFilters.add(filter);
    }
    if (filter is BoolFilter) {
      boolFilters.add(filter);
    }
    if (filter is DateFilter) {
      dateFilters.add(filter);
    }
    if (filter is EnumFilter) {
      enumFilters.add(filter);
    }
    return this;
  }

  Map<String, dynamic> build() {
    Map<String, dynamic> multiKeyMap = MultiKeyMap();
    for (var filter in textFilters) {
      multiKeyMap.addAll(filter.build());
    }
    for (var filter in numberFilters) {
      multiKeyMap.addAll(filter.build());
    }
    for (var filter in boolFilters) {
      multiKeyMap.addAll(filter.build());
    }
    for (var filter in dateFilters) {
      multiKeyMap.addAll(filter.build());
    }
    for (var filter in enumFilters) {
      multiKeyMap.addAll(filter.build());
    }
    return multiKeyMap;
  }

  @override
  Map<String, dynamic> toJson() => _$FiltersToJson(this);
  @override
  Filters? fromJsonMap(Map<String, dynamic>? json) =>
      json != null ? Filters.fromJson(json) : null;
  factory Filters.fromJson(Map<String, dynamic> json) =>
      _$FiltersFromJson(json);
}
