enum SortParamMode {
  ASC,
  DESC,
}

class SortParamUtils {
  static const String SORT_KEY = 'sort';
  static const String SORT_SEPARATOR = ',';

  static Map<String, dynamic> build(
      {String sortKey = SORT_KEY,
      String separator = SORT_SEPARATOR,
      required List<SortParamData> sortParams}) {
    if (sortParams.isEmpty) return {};
    return {
      sortKey: sortParams
          .map((sortParam) => sortParam.build(separator: separator))
          .toSet()
    };
  }
}

class SortParamData {
  String param;
  SortParamMode sortMode;

  SortParamData({required this.param, SortParamMode? sortMode})
      : sortMode = sortMode ?? SortParamMode.ASC;

  String? get sortModeName => sortMode.name;

  String build({String separator = SortParamUtils.SORT_SEPARATOR}) =>
      '$param$separator${sortMode.name}';

  static SortParamData fromMap(
      {String sortKey = SortParamUtils.SORT_KEY,
      String separator = SortParamUtils.SORT_SEPARATOR,
      Map<String, dynamic>? map,
      required String defaultParam,
      SortParamMode? defaultSortMode}) {
    SortParamData defaultSortParam =
        SortParamData(param: defaultParam, sortMode: defaultSortMode);
    if (map == null || map.isEmpty) return defaultSortParam;
    SortParamData? sortParam = fromString(map[sortKey]);
    return sortParam ?? defaultSortParam;
  }

  static SortParamData? fromString(String? data) {
    if (data == null) return null;
    SortParamMode? sortMode;
    if (data.toUpperCase().endsWith(SortParamMode.ASC.name)) {
      sortMode = SortParamMode.ASC;
    } else if (data.toUpperCase().endsWith(SortParamMode.DESC.name)) {
      sortMode = SortParamMode.DESC;
    }
    if (sortMode == null) return null;
    String param = data.substring(0, (data.length - sortMode.name.length) - 1);
    return SortParamData(param: param, sortMode: sortMode);
  }
}
