import 'package:adeptarch_base/src/widgets/widgets.dart';
import 'package:adeptarch_base/src/filters/filters.dart';
import 'package:flutter/material.dart';

// ignore_for_file: must_be_immutable
class BoolFilterView extends BaseFilterView<BoolFilter> {
  String textUnspecified, textTrue, textFalse;

  BoolFilterView({
    FilterController<BoolFilter>? filterController,
    String? hint,
    bool? allowSorting,
    this.textUnspecified = "Unspecified",
    this.textTrue = "Yes",
    this.textFalse = "No",
  }) : super(
          filterController: filterController,
          hint: hint,
          allowSorting: allowSorting,
        );

  @override
  State createState() => _BoolFilterViewState();
}

class _BoolFilterViewState extends State<BoolFilterView> {
  late Color colorPrimary, colorTextTertiary;
  late List<DropdownMenuItem<String>> boolFilterValues;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    generateFilterItems();
    onFilterChanged();
  }

  /// DropDownMenuItem of Strings is used instead of bools because null value
  /// is not supported by DropdownButton, when a null value is used then
  /// no text value is shown.
  void generateFilterItems() {
    boolFilterValues = [
      DropdownMenuItem<String>(
        value: 'null',
        child: Text(widget.textUnspecified),
      ),
      DropdownMenuItem<String>(
        value: 'true',
        child: Text(widget.textTrue),
      ),
      DropdownMenuItem<String>(
        value: 'false',
        child: Text(widget.textFalse),
      ),
    ];
  }

  void listeners() {
    widget.filterController.addListener(onFilterChanged);
  }

  void onFilterChanged() {
    BoolFilter? filter = widget.filterController.value;
    if (filter != null) {}
  }

  void checkColors(context) {
    colorPrimary = Theme.of(context).primaryColor;
    colorTextTertiary = Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
        if (widget.hint != null)
          Text(
            widget.hint!,
            style: TextStyle(
                color: widget.filterController.value?.value != null
                    ? colorPrimary
                    : colorTextTertiary,
                fontSize: 12),
          ),
        Row(
          children: <Widget>[
            Expanded(
              child: Theme(
                data: ThemeData(brightness: Theme.of(context).brightness),
                child: DropdownButton<String>(
                  isExpanded: true,
                  items: boolFilterValues,
                  value: widget.filterController.value?.value?.toString() ??
                      'null',
                  onChanged: (value) {
                    if (value == 'null') {
                      widget.filterController.reset();
                    } else {
                      widget.filterController.value?.equals(value == 'true');
                      widget.filterController.clearError();
                    }
                    setState(() {});
                  },
                ),
              ),
            ),
            Visibility(
              visible: widget.allowSorting,
              child: Container(
                margin: const EdgeInsets.only(top: 16),
                alignment: Alignment.bottomCenter,
                child: SortView(
                  sortMode: widget.filterController.value?.sortMode,
                  listener: (oldMode, newMode) {
                    widget.filterController.value?.sortMode = newMode;
                  },
                ),
              ),
            ),
          ],
        ),
      ],
    );
  }

  @override
  void dispose() {
    widget.filterController.removeListener(onFilterChanged);
    super.dispose();
  }
}
